/**

 Implementation Canvas class
 Fraga 03/09/2002 
 Fraga 16/09/2004. Documenting Canvas::MidPointLine 


**/

#include <qpainter.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "canvas.h"


Canvas::Canvas( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
    ang = 0;
    setPalette( QPalette( QColor( 250, 250, 200) ) );
	setMouseTracking( TRUE );
	px = py = 0;
}


void Canvas::setAngle( int degrees )
{
    if ( degrees < 5 )
	degrees = 5;
    if ( degrees > 70 )
	degrees = 70;
    if ( ang == degrees )
	return;
    ang = degrees;
    repaint();
//    emit angleChanged( ang );
}


void Canvas::paintEvent( QPaintEvent * )
{
    QString s = "Angle = " + QString::number( ang );
    QPainter p( this );
    p.drawText( w/2 + 140, h/2, s );

	testLines( &p );
}

void Canvas::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );  

	px = c.x( );
	py = c.y( );
	if ( px>=0 && py>=0 )  emit changePos ( );
}

void Canvas::resizeEvent ( QResizeEvent * )
{
	w = width( );
	h = height( );
}

QSizePolicy Canvas::sizePolicy() const
{
    return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

/** Drawing primitives **/
void Canvas::MidPointLine( int x0, int y0, int x1, int y1, QPainter *p )
{
	int tmp;
	int dx, dy;
	int d, incrE, incrNE;
	int x, y;

	if ( y1 < y0 ) {  /** Swap points **/
		tmp = y0; y0 = y1; y1 = tmp;
		tmp = x0; x0 = x1; x1 = tmp;
	}

	dx = abs(x1 - x0);
	dy = y1 - y0;

	if ( dx == 0 && dy == 0 ) return;

	if ( dy < dx ) {     /** m<1 **/
		d = 2*dy - dx;
		incrE = 2*dy;
		incrNE = 2*(dy - dx);
	}
	else {               /** m>1 **/
		d = 2*dx - dy;
		incrE = 2*dx;
		incrNE = 2*(dx - dy);
	}
	x = x0;
	y = y0; 

	if ( dx == 0 ) {
		if ( dy == 0 ) return;  // patological case

		for( y=y0; y<=y1; y++ ) // case 3
        	p->drawPoint( x0, y ); 
	}
	else if ( dy == 0 ) {
		if ( x0 > x1 ) { /** swap x's. Case 5  **/
			tmp = x0; x0 = x1; x1 = tmp;
		}
		for( x=x0; x<=x1; x++ )   // case 1
        	p->drawPoint( x, y0 ); 
	}
	else if ( dx == dy ) {
		if ( x0 < x1 ) {  // case 2 
			for( x=x0, y=y0; x<=x1; x++, y++ )
       		 	p->drawPoint( x, y ); 
		}
		else { // case 4
			for( x=x0, y=y0; x>=x1; x--, y++ )
       		 	p->drawPoint( x, y ); 
		}
	}
	else if ( x0 < x1 ) {
       	p->drawPoint( x, y ); 

		if ( dy < dx ) {  /** Normal algorithm. Case 9 **/
			while ( x < x1 ) {
				if ( d <= 0 )
					d += incrE;
				else {
					y++;
					d += incrNE;
				} 
				x++;
       			p->drawPoint( x, y ); 
			}
		}
		else {  /** Case 10  **/
			while ( y < y1 ) {
				if ( d <= 0 )
					d += incrE;
				else {
					x++;
					d += incrNE;
				} 
				y++;
       			p->drawPoint( x, y ); 
			}
		}
	}
	else {
       	p->drawPoint( x, y ); 
		if ( dy > dx ) {
			while ( y < y1 ) {  // Case 11
				if ( d <= 0 )
					d += incrE;
				else {
					x--;
					d += incrNE;
				}
				y++; 
       			p->drawPoint( x, y ); 
			}
		}
		else {                // Case 12
			while ( x > x1 ) {
				if ( d <= 0 )
					d += incrE;
				else {
					y++;
					d += incrNE;
				} 
				x--;
       			p->drawPoint( x, y ); 
			}
		}
	}
}

void Canvas::testLines( QPainter *p )
{
	int cx = w/2; // canvas center
	int cy = h/2; // canvas center
	int incrang = 5;
	int l = 150;

	int n = 360/incrang;
	// int n = 19;

	for ( int i=0; i<n; i++ ) {
		float ang = i*incrang;

		ang = ang * ANGTORAD;

		float val = l*cos(ang) + cx;
		int px = Round( val );
		
		val = l*sin(ang) + cy;
		int py = Round( val );

		MidPointLine( cx, cy, px, py, p );
	}
}

